# RpiL/PWM

import RPi.GPIO as GPIO

GPIO.setmode(GPIO.BCM)

class PWM:
    def __init__(self, pin, frequency=1000, duty_cycle=50):
        """This uses GPIO pin-number on the pi.\n
        This class is for Pulse Width Modulation"""
        self.pin = int(pin)
        self.frequency = frequency
        self.duty_cycle = duty_cycle

        GPIO.setup(self.pin, GPIO.OUT)
        self.pwm = GPIO.PWM(self.pin, self.frequency)
        self.pwm_started = False

    def start(self):
        """Start the PWM with specified Duty-Cycle"""
        if not self.pwm_started:
            self.pwm.start(self.duty_cycle)
            self.pwm_started = True
        else:
            self.pwm.ChangeDutyCycle(self.duty_cycle)

    def change_duty_cycle(self, duty_cycle):
        if self.pwm_started:
            self.pwm.ChangeDutyCycle(duty_cycle)
        self.duty_cycle = duty_cycle

    def change_frequency(self, frequency):
        if self.pwm_started:
            self.stop()
            self.pwm = GPIO.PWM(self.pin, frequency)
            self.pwm.start(self.duty_cycle)
            self.pwm_started = True
        self.frequency = frequency

    def stop(self):
        if self.pwm_started:
            self.pwm.stop()
            self.pwm_started = False

    def __del__(self):
        self.stop()
        GPIO.cleanup([self.pin])



import RPi.GPIO as GPIO
import time as t
import threading

GPIO.setmode(GPIO.BCM)

class RGB_LED:
    def __init__(self, red_pin, green_pin, blue_pin):
        self.red_pin = red_pin
        self.green_pin = green_pin
        self.blue_pin = blue_pin

        GPIO.setup(self.red_pin, GPIO.OUT)
        GPIO.setup(self.green_pin, GPIO.OUT)
        GPIO.setup(self.blue_pin, GPIO.OUT)

        self.red_pwm = GPIO.PWM(self.red_pin, 100)
        self.green_pwm = GPIO.PWM(self.green_pin, 100)
        self.blue_pwm = GPIO.PWM(self.blue_pin, 100)

        self.red_pwm.start(0)
        self.green_pwm.start(0)
        self.blue_pwm.start(0)

        self.running = False
        self.rainbow_thread = None

    def set_color(self, hex_color):
        """Set the RGB LED to a specific hex color."""
        try:
            hex_color = hex_color.lstrip('#')
            red = int(hex_color[0:2], 16)
            green = int(hex_color[2:4], 16)
            blue = int(hex_color[4:6], 16)

            # Convert RGB values (0-255) to PWM duty cycle (0-100)
            self.red_pwm.ChangeDutyCycle(red / 255 * 100)
            self.green_pwm.ChangeDutyCycle(green / 255 * 100)
            self.blue_pwm.ChangeDutyCycle(blue / 255 * 100)
        except ValueError:
            print("Invalid hex color format. Please use a hex string (e.g., '#FF00FF').")

    def rainbow_cycle_sequence(self, wait=0.05):
        """Cycle through colors in a rainbow effect until stopped."""
        self.running = True
        position = 0  # Start at the beginning of the color wheel
        while self.running:
            red, green, blue = self.wheel(position % 256)
            self.red_pwm.ChangeDutyCycle(red / 255 * 100)
            self.green_pwm.ChangeDutyCycle(green / 255 * 100)
            self.blue_pwm.ChangeDutyCycle(blue / 255 * 100)
            t.sleep(wait)
            position += 1

    def rainbow_cycle(self, wait=0.05):
        """Start the rainbow cycle in a separate thread if not already running."""
        if self.rainbow_thread is None or not self.rainbow_thread.is_alive():
            self.rainbow_thread = threading.Thread(target=self.rainbow_cycle_sequence, args=(wait,))
            self.rainbow_thread.start()

    def stop_rainbow_cycle(self):
        """Stop the rainbow cycle."""
        self.running = False
        self.off()

    @staticmethod
    def wheel(position=0):
        """Generate rainbow colors across 0-255 positions."""
        if position < 85:
            return (position * 3, 255 - position * 3, 0)
        elif position < 170:
            position -= 85
            return (255 - position * 3, 0, position * 3)
        else:
            position -= 170
            return (0, position * 3, 255 - position * 3)

    def off(self):
        """Turn off the RGB LED."""
        self.red_pwm.ChangeDutyCycle(0)
        self.green_pwm.ChangeDutyCycle(0)
        self.blue_pwm.ChangeDutyCycle(0)

    def __del__(self):
        """Cleanup GPIO pins when the object is deleted."""
        self.off()
        self.red_pwm.stop()
        self.green_pwm.stop()
        self.blue_pwm.stop()
        if self.rainbow_thread and self.rainbow_thread.is_alive():
            self.rainbow_thread.join()
        GPIO.cleanup(self.red_pin, self.green_pin, self.blue_pin)



# RpiL/CPU

from gpiozero import CPUTemperature

class CPU:
    def __init__(self):
        self.self = self
        self.cpu = CPUTemperature()

    def temperature(self, measure="C"):
        """Measure should be in F-Fahrenheit, C-Celsius, K-Kelvin"""
        if measure == "C":
            return self.cpu.temperature()
        elif measure == "F":
            return self.cpu.temperature() * (9 / 5 + 32)
        elif measure == "K":
            return self.cpu.temperature() + 273.15
        else:
            raise ValueError("measure must be C-Celsius, F-Fahrenheit, K-Kelvin")

    def __del__(self):
        self.self = None


# RpiL/pin

import RPi.GPIO as GPIO

GPIO.setmode(GPIO.BCM)

class pin:
    def __init__(self, pin, mode):
        """This uses GPIO pin-number on the pi.\n
        This class is used to turn a gpio pin on or off.
        Use 'IN' or 'OUT' for input or output mode"""
        self.pin = int(pin)
        if mode.upper() == 'IN':
            self.mode = GPIO.IN
        else:
            self.mode = GPIO.OUT

        GPIO.setup(self.pin, self.mode)

    def on(self):
        if self.mode == GPIO.OUT:
            GPIO.output(self.pin, GPIO.HIGH)

    def off(self):
        if self.mode == GPIO.OUT:
            GPIO.output(self.pin, GPIO.LOW)

    def toggle(self):
        if self.mode == GPIO.OUT:
            if GPIO.input(self.pin) == 0:
                self.on()
            else:
                self.off()

    def value(self):
        """Return the gpio pin state (on/off, 1/0)."""
        if self.mode == GPIO.IN:
            return GPIO.input([self.pin])



# RpiL/PIR

import RPi.GPIO as GPIO
import threading
import time as t

GPIO.setmode(GPIO.BCM)

class PIR:
    def __init__(self, pin):
        self.pin = pin
        self.motion_detected_flag = False
        self.motion_thread = None

        GPIO.setup(pin, GPIO.IN)

        self.motion_thread = threading.Thread(target=self._monitor_motion)
        self.motion_thread.daemon = True
        self.motion_thread.start()

    def _monitor_motion(self):
        while True:
            if GPIO.input(self.pin):
                self.motion_detected_flag = True
            else:
                self.motion_detected_flag = False
            t.sleep(0.25)

    def motion_detected(self):
        return self.motion_detected_flag

    def wait_for_motion(self):
        """Returns a Boolean when motion is detected"""
        while not self.motion_detected_flag:
            t.sleep(0.25)
        return True

    def __del__(self):
        GPIO.cleanup([self.pin])


from .LED import LED
from .Motor_Driver import Motor_Driver
from .PWM import PWM
from .RGB_LED import RGB_LED
from .buzzer import buzzer
from .motor import motor
from .pin import pin
from .USDS import USDS
from .Temperature_Sensor import DHT11, DHT22, AM2302
from .PIR import PIR
from .IR import IR_Receiver, IR_LED



# RpiL/LED

import RPi.GPIO as GPIO
import time as t
import threading

GPIO.setmode(GPIO.BCM)

class LED:
    def __init__(self, pin):
        """This uses GPIO pin-number on the pi.\n
        This class is for controlling basic LED's."""
        self.pin = int(pin)

        self.led_thread = None

        GPIO.setup(self.pin, GPIO.OUT)

    def on(self):
        if GPIO.input(self.pin) == 0:
            GPIO.output(self.pin, GPIO.HIGH)

    def off(self):
        if GPIO.input(self.pin) == 1:
            GPIO.output(self.pin, GPIO.LOW)

    def toggle(self):
        if GPIO.input(self.pin) == 0:
            self.on()
        else:
            self.off()

    def blink_sequence(self, duration=6):
        GPIO.output(self.pin, GPIO.LOW)
        for i in range(int(duration)):
            self.on()
            t.sleep(0.5)
            self.off()
            t.sleep(0.5)

    def blink(self, duration=6):
        if self.led_thread and self.led_thread.is_alive():
            self.led_thread.join()

        self.led_thread = threading.Thread(target=self.blink_sequence, args=(duration,))
        self.led_thread.start()

    def __del__(self):
        GPIO.output(self.pin, GPIO.LOW)
        if self.led_thread and self.led_thread.is_alive():
            self.led_thread.join()
        GPIO.cleanup([self.pin])



# RpiL/buzzer

import RPi.GPIO as GPIO
import time as t
import threading

GPIO.setmode(GPIO.BCM)

class buzzer:
    """This uses GPIO pin-number on the pi.\n
    This class is for controlling basic Piezo Buzzers."""
    def __init__(self, pin):
        self.pin = pin

        self.beep_thread = None

        GPIO.setup(pin, GPIO.OUT)

    def on(self):
        if GPIO.input(self.pin) == 0:
            GPIO.output(self.pin, GPIO.HIGH)

    def off(self):
        if GPIO.input(self.pin) == 1:
            GPIO.output(self.pin, GPIO.LOW)

    def toggle(self):
        if GPIO.input(self.pin) == 0:
            self.on()
        else:
            self.off()

    def beep_sequence(self, duration):
        GPIO.output(self.pin, GPIO.LOW)
        for i in range(int(duration)):
            self.on()
            t.sleep(0.5)
            self.off()
            t.sleep(0.5)

    def beep(self, duration):
        """Uses threads to make Beep() a non-blocking function"""
        if self.beep_thread and self.beep_thread.is_alive():
            self.beep_thread.join()

        self.beep_thread = threading.Thread(target=self.beep_sequence, args=(duration,))
        self.beep_thread.start()

    def __del__(self):
        GPIO.output(self.pin, GPIO.LOW)
        if self.beep_thread and self.beep_thread.is_alive():
            self.beep_thread.join()
        GPIO.cleanup(self.pin)



# RpiL/USDS

import RPi.GPIO as GPIO
import time as t

GPIO.setmode(GPIO.BCM)

class USDS:
    def __init__(self, trig, echo):
        """This uses GPIO pin-numbers on the pi./n
        USDS stands for Ultra-Sonic Distance Sensor"""
        self.trig = trig
        self.echo = echo

        GPIO.setup(self.trig, GPIO.OUT)
        GPIO.setup(self.echo, GPIO.IN)

    def distance(self):
        """Measures distance in cm"""
        GPIO.output(self.trig, GPIO.LOW)
        t.sleep(0.5)

        GPIO.output(self.trig, GPIO.HIGH)
        t.sleep(0.00001)
        GPIO.output(self.trig, GPIO.LOW)

        start = t.time()
        while GPIO.input(self.echo) == 0:
            start = t.time()

        stop = t.time()
        while GPIO.input(self.echo) == 1:
            stop = t.time()

        elapsed = stop - start
        distance_ = (elapsed * 34300) / 2

        return distance_

    def __del__(self):
        GPIO.cleanup([self.trig, self.echo])


# RpiL/IR

import RPi.GPIO as GPIO
import pigpio
import time as t

GPIO.setmode(GPIO.BCM)

class IR_Receiver:
    def __init__(self, pin):
        self.pin = pin
        self.pi = pigpio.pi()

        if not self.pi.connected:
            raise Exception('could not connect to pigpio daemon')

        self.pi.set_mode(self.pin, pigpio.INPUT)
        self.code= []

        self.callback = self.pi.callback(self.pin, pigpio.EITHER_EDGE, self._callback)

    def _callback(self, pin, level, tick):
        self.code.append(tick)

    def _get_signal(self):
        t.sleep(0.1)

        if len(self.code) > 0:
            pulse_data = self.code
            self.code = []

            return pulse_data
        else:
            return None

    def read_signal(self, protocal="NEC"):
        pulse_data = self._get_signal()
        if pulse_data:
            if protocal == "NEC":
                return self._decode_nec(pulse_data)
            else:
                return ValueError("Unsupported protocol")
        return None

    def _decode_nec(self, pulse_data):
        bits = []
        for i in range(1, len(pulse_data), 2):
            high_time = pulse_data[i] - pulse_data[i-1]
            low_time = pulse_data[i+1] - pulse_data[i]
            if high_time > 900:
                bits.append(1)
            else:
                bits.append(0)

        hex_value = ''.join(str(b) for b in bits)
        hex_code = hex(int(hex_value, 2))
        return hex_code

    def __del__(self):
        self.callback.cancel()
        self.pi.stop()
        GPIO.cleanup([self.pin])

class IR_LED:
    def __init__(self, pin):
        self.pin = pin
        self.pi = pigpio.pi()

        if not self.pi.connected:
            raise Exception('could not connect to pigpio daemon')

        self.pi.set_mode(self.pin, pigpio.OUTPUT)

    def send_signal(self, hex_code, protocol="NEC", frequency=38000):
        if protocol == "NEC":
            data = self._hex_to_bin(hex_code)
            self._send_nec(data, frequency)
        else:
            raise Exception('unsupported protocol')

    def _send_nec(self, data, frequency=38000):
        self.pi.set_PWM_frequency(self.pin, frequency)
        self.pi.set_PWM_dutycycle(self.pin, 128)

        self._send_pulse(9000, 4500)

        for i in range(32):
            bit = (data >> (31 - i)) & 1
            if bit == 1:
                self._send_pulse(560, 1690)
            else:
                self._send_pulse(560, 560)

        self.pi.set_PWM_dutycycle(self.pin, 0)

    def _send_pulse(self, high_time, low_time):
        self.pi.gpio_trigger(self.pin, high_time, 1)
        t.sleep(low_time / 1000000.0)

    def _hex_to_bin(self, hex_code):
        hex_code = hex_code.lstrip("0x").zfill(8)
        bin_code = bin(int(hex_code, 16))[2:].zfill(32)
        return bin_code

    def __del__(self):
        self.pi.stop()
        GPIO.cleanup([self.pin])


# RpiL/motor

import RPi.GPIO as GPIO

GPIO.setmode(GPIO.BCM)

class motor:
    def __init__(self, forward_pin, backward_pin):
        """This uses GPIO pin-numbers for the pi.\n
        This class is to run a motor without a driver"""
        self.forward_pin = forward_pin
        self.backward_pin = backward_pin

        GPIO.setup(self.forward_pin, GPIO.OUT)
        GPIO.setup(self.backward_pin, GPIO.OUT)

    def forward(self):
        GPIO.output(self.forward_pin, GPIO.HIGH)
        GPIO.output(self.backward_pin, GPIO.LOW)

    def backward(self):
        GPIO.output(self.forward_pin, GPIO.LOW)
        GPIO.output(self.backward_pin, GPIO.HIGH)

    def stop(self):
        GPIO.output(self.forward_pin, GPIO.LOW)
        GPIO.output(self.backward_pin, GPIO.LOW)

    def __del__(self):
        GPIO.cleanup([self.forward_pin, self.backward_pin])



from setuptools import setup, find_packages

setup(
    name="RpiL",
    version="0.2.2.2",
    packages=find_packages(where="src"),  # finds 'rpi_gpio' inside 'src'
    package_dir={"": "src"},
    install_requires=["pigpio", "gpiozero"],
    extras_require={
        "rpi": ["RPi.GPIO", "Adafruit_DHT"]
    },
    description="Library for controlling Raspberry Pi hardware.",
    author="Zevi Berlin",
    author_email="zeviberlin@gmail.com",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    long_description="""
        RpiL is a Python library for controlling Raspberry Pi hardware.

        For detailed documentation, visit:
        `Documentation <https://rpil.readthedocs.io>`_

        The source code is available on GitHub:
        `GitHub Repository <https://github.com/TrynaThinkOf1/RpiL>`_
        """,
    long_description_content_type="text/x-rst",
    url="https://rpil.readthedocs.io",
)



# RpiL/Heat_Sensor

import RPi.GPIO as GPIO
import Adafruit_DHT as DHT

GPIO.setmode(GPIO.BCM)

class Temperature_Sensor:
    def __init__(self, out, sensor_type):
        """This uses GPIO pin-numbers on the pi."""
        self.out = out
        self.sensor_type = sensor_type

        GPIO.setup(self.out, GPIO.IN)

    def temperature(self, measure_mode="C"):
        """measure_mode is for:/n
        'F', 'Fahrenheit', 'C', 'Celsius', 'Centigrade'"""
        humidity, temp = DHT.read_retry(self.sensor_type, self.out)

        if temp is not None:
            if measure_mode in ['F', 'Fahrenheit']:
                return temp * 9.0 / 5.0 + 32.0
            elif measure_mode in ['C', 'Celsius', 'Centigrade']:
                return temp
        else:
            return None

    def humidity(self):
        humidity, _ = DHT.read_retry(self.sensor_type, self.out)

        if humidity is not None:
            return humidity
        else:
            return None

class DHT11(Temperature_Sensor):
    def __init__(self, out):
        super().__init__(out, DHT.DHT11)

class DHT22(Temperature_Sensor):
    def __init__(self, out):
        super().__init__(out, DHT.DHT22)

class AM2302(Temperature_Sensor):
    def __init__(self, out):
        super().__init__(out, DHT.AM2302)



# RpiL/Motor_Driver

import RPi.GPIO as GPIO

GPIO.setmode(GPIO.BCM)

class Motor_Driver:
    def __init__(self, in1=0, in2=0, ena=0, in3=0, in4=0, enb=0):
        """This uses GPIO pin-number on the pi.\n
        This class is for the L298N Motor Driver"""
        self.in1 = int(in1)
        self.in2 = int(in2)
        self.ena = int(ena)

        self.in3 = int(in3)
        self.in4 = int(in4)
        self.enb = int(enb)

        GPIO.setup(self.in1, GPIO.OUT)
        GPIO.output(self.in1, GPIO.LOW)
        GPIO.setup(self.in2, GPIO.OUT)
        GPIO.output(self.in2, GPIO.LOW)

        GPIO.setup(self.ena, GPIO.OUT)

        GPIO.setup(self.in3, GPIO.OUT)
        GPIO.output(self.in3, GPIO.LOW)
        GPIO.setup(self.in4, GPIO.OUT)
        GPIO.output(self.in4, GPIO.LOW)

        GPIO.setup(self.enb, GPIO.OUT)

        self.en_a = GPIO.PWM(self.ena, 100)
        self.en_a.start(0)

        self.en_b = GPIO.PWM(self.enb, 100)
        self.en_b.start(0)

    def forward(self, speed=90):
        """Turns IN1 and IN3 on"""
        GPIO.output(self.in1, GPIO.HIGH)
        GPIO.output(self.in2, GPIO.LOW)
        self.en_a.ChangeDutyCycle(speed)

        GPIO.output(self.in3, GPIO.HIGH)
        GPIO.output(self.in4, GPIO.LOW)
        self.en_b.ChangeDutyCycle(speed)

    def backward(self, speed=75):
        """Turns IN2 and IN4 on"""
        GPIO.output(self.in1, GPIO.LOW)
        GPIO.output(self.in2, GPIO.HIGH)
        self.en_a.ChangeDutyCycle(speed)

        GPIO.output(self.in3, GPIO.LOW)
        GPIO.output(self.in4, GPIO.HIGH)
        self.en_b.ChangeDutyCycle(speed)

    def turn_right(self, speed=90):
        """Turns IN2 and IN3 on"""
        GPIO.output(self.in1, GPIO.LOW)
        GPIO.output(self.in2, GPIO.HIGH)
        self.en_a.ChangeDutyCycle(speed)

        GPIO.output(self.in3, GPIO.HIGH)
        GPIO.output(self.in4, GPIO.LOW)
        self.en_b.ChangeDutyCycle(speed)

    def turn_left(self, speed=90):
        """Turns IN1 and IN4 on"""
        GPIO.output(self.in1, GPIO.HIGH)
        GPIO.output(self.in2, GPIO.LOW)
        self.en_a.ChangeDutyCycle(speed)

        GPIO.output(self.in3, GPIO.LOW)
        GPIO.output(self.in4, GPIO.HIGH)
        self.en_b.ChangeDutyCycle(speed)

    def stop(self):
        """Turns all IN# off and turns off ena and enb PWM"""
        GPIO.output(self.in1, GPIO.LOW)
        GPIO.output(self.in2, GPIO.LOW)
        self.en_a.stop()

        GPIO.output(self.in3, GPIO.LOW)
        GPIO.output(self.in4, GPIO.LOW)
        self.en_b.stop()

    def __del__(self):
        self.stop()
        GPIO.cleanup([self.in1, self.in2, self.ena, self.in3, self.in4, self.enb])
